/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.seal.pki;

import java.security.cert.X509Certificate;
import java.util.List;
import java.util.Properties;

import dk.sosi.seal.pki.impl.HashMapCertificateCache;
import dk.sosi.seal.pki.impl.PropertiesSOSIConfiguration;
import dk.sosi.seal.pki.impl.federationcert.FederationCertificateStoreAdapter;

/**
 * Implementation of CertificationAuthority representing the OCES production CA.
 *
 * @author ads@lakeside.dk
 * @since 2.0
 */
public class OCESCertificationAuthority extends AbstractOCESCertificationAuthority {

	private static final String OCES3_ROOT_CERTIFICATE_BASE_64 =
            "MIIGfDCCBDCgAwIBAgIUfdkPhrP/1uxz0JMLkAP8WU8ICm8wQQYJKoZIhvcNAQEK\n" +
                    "MDSgDzANBglghkgBZQMEAgMFAKEcMBoGCSqGSIb3DQEBCDANBglghkgBZQMEAgMF\n" +
                    "AKIDAgFAME0xJDAiBgNVBAMMG0RlbiBEYW5za2UgU3RhdCBPQ0VTIHJvZC1DQTEY\n" +
                    "MBYGA1UECgwPRGVuIERhbnNrZSBTdGF0MQswCQYDVQQGEwJESzAeFw0yMTExMDkx\n" +
                    "MjM3MjdaFw00NjExMDMxMjM3MjZaME0xJDAiBgNVBAMMG0RlbiBEYW5za2UgU3Rh\n" +
                    "dCBPQ0VTIHJvZC1DQTEYMBYGA1UECgwPRGVuIERhbnNrZSBTdGF0MQswCQYDVQQG\n" +
                    "EwJESzCCAiIwDQYJKoZIhvcNAQEBBQADggIPADCCAgoCggIBAM1z8k+VAdXUmS9u\n" +
                    "RwPRX2o/vp1F27kjJhzpBtiq87Cncw2jmNjkSXA2KmDL/bfMefLBc9PTRKcufCeB\n" +
                    "6+nb0yJvj7vKhMRkOcbIH4Y0Jn67KBtAWiCN2eINVQgaVX5fmMwJjWNj2aMXG5WY\n" +
                    "HZSSgMSZkiBf3NXeEU8Skg4eqN8qVisR5Rhq9h/MOgD1iPPalgidUG8J7tvxvkHM\n" +
                    "4BA1BbPCtuXH/oTl1pHouTn8SERX2OMQH5oVa1xXo3S33VS/YRydurACIepgALta\n" +
                    "ARKxaQ0Q6InzBJno/Ky8FlM5Pejq5uIlPnN2UjNgYoy5vd+S30eR9ys855fQo/AE\n" +
                    "bJuXK+7PqKs/bykuvkC2obupRfpt6PCCSwidgSqazMl5hoQxzqOVL6KLK+SaS/+f\n" +
                    "KB/t8sQlSXaXOc37PRbE4UZ1useyvmiLDCk4xlkNud8dY9gdg9Bzuad/uXybCJ3x\n" +
                    "PpSyaSotiLQ0qSrU5d7Zm4gaWPamKK0La+pi/LVcslStQKaKvZWIWGAsHP+PSd3q\n" +
                    "cRLhKkSLh7TD6bRrNdS40HC3K2rlkHWpclHHx4DVZx10vR2+DL5XJCMQXqanp/4w\n" +
                    "aJB5Sfw5BJVaYatbY3h+3sWIYVDK/gKj+DjHxGuXP3mTvx4QhJQAbxXsYe9LD0OU\n" +
                    "bTrkIITKVmmFibgnMvIURiVAc8EHAgMBAAGjgeswgegwDwYDVR0TAQH/BAUwAwEB\n" +
                    "/zBrBggrBgEFBQcBAQRfMF0wNwYIKwYBBQUHMAKGK2h0dHA6Ly9jYTEuZ292LmRr\n" +
                    "L29jZXMvcm9vdC9jYWNlcnQvcm9vdC5jZXIwIgYIKwYBBQUHMAGGFmh0dHA6Ly9j\n" +
                    "YTEuZ292LmRrL29jc3AwOQYDVR0fBDIwMDAuoCygKoYoaHR0cDovL2NhMS5nb3Yu\n" +
                    "ZGsvb2Nlcy9yb290L2NybC9yb290LmNybDAdBgNVHQ4EFgQUM2UbcJY08Augh5yD\n" +
                    "3kxPiAytbrYwDgYDVR0PAQH/BAQDAgEGMEEGCSqGSIb3DQEBCjA0oA8wDQYJYIZI\n" +
                    "AWUDBAIDBQChHDAaBgkqhkiG9w0BAQgwDQYJYIZIAWUDBAIDBQCiAwIBQAOCAgEA\n" +
                    "IYh5cL3Ltq5FgWxAzdPyGJZ4YK8c0ud9riLm27LQmyZEXfddilNImIXq36HpDkdc\n" +
                    "RDbI76hJWsUn2fc5F8BY3hSDEkXBmC2YMaDPGmvWDsVxn5wIjdmX3nTvvpkR99BZ\n" +
                    "EcngOYnOlZH/pmL/7OXVjMO/H3MAnzSo91sNlQgHsgKkqVxltVflNvNYkvRAN+Ea\n" +
                    "v5QejsSuZ+kQKHGqMj9AqFc9wC2OCeLsuh/Z3UlgrlVdlxJUZMMP3CEfNpMIVR/t\n" +
                    "VEMzRxkhuYM4NHRDYIu5kjg5w4hJw6K+K8MECoowqgjf9X4dy03HpsLODOVaC+u5\n" +
                    "nM9BknjY0YBi7FDk0NHjPOwFYJoQ3reoZoYWYJ+dWFyQGJoR+FW4nMu6bZbGjsUA\n" +
                    "BSHPqV/cPdC4eVVW9LJZlCHCaGrMdmF2I0wZEqnpfS2kIDgLOWbazG22pBDno3QX\n" +
                    "EHkbTEYWsubkkAsys9/3RIZPlS3ZrFXsPf3VGeXzqTE00WpcJ6P4ffMRyLjXosXG\n" +
                    "3lPMP4eLGml3GfCigEX4RNDZUsA2bv56Od/gzAcKKap3nIgC08MY4zj1OXvrXEEr\n" +
                    "r0CTEI2mqAxN/hYmXVXlsk3Mx8MSm/eYTAzKcAx7wErJvZDrceGUjfe07Z7iXSEk\n" +
                    "wUrknahrmVWcFbzaDuX0p8tbuU5eJs2OTdTkAuiFfIY=";

    public static final List<X509Certificate> OCES_3_ROOT_CERTIFICATES = asCertificates(OCES3_ROOT_CERTIFICATE_BASE_64);
    
    /**
     * Constructor for the <code>OCESCertificationAuthority</code> class.
     *
     * @param properties
     *            The initialization <code>Properties</code> of the system
     */
    @Deprecated
    public OCESCertificationAuthority(Properties properties, CertificateStatusChecker certificateStatusChecker, IntermediateCertificateCache intermediateCertificateCache) {
        // This is not nice at all! To be removed in next Seal release
        this(PropertiesSOSIConfiguration.createWithDefaultOcesProperties(properties), certificateStatusChecker, intermediateCertificateCache,
                new FederationCertificateStoreAdapter(PropertiesSOSIConfiguration.createWithDefaultOcesProperties(properties), new HashMapCertificateCache()));
    }

    public OCESCertificationAuthority(SOSIConfiguration configuration, CertificateStatusChecker certificateStatusChecker, IntermediateCertificateCache intermediateCertificateCache, FederationCertificateResolver federationCertificateResolver) {
        super(configuration, certificateStatusChecker, intermediateCertificateCache, federationCertificateResolver);
    }

    @Override
    protected List<X509Certificate> getOCES3RootCertificates() {
        return OCES_3_ROOT_CERTIFICATES;
    }

    protected String getCertificationAuthorityName() {
        return "OCES Production";
    }
}