/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.seal.pki;

import java.security.cert.X509Certificate;
import java.util.List;
import java.util.Properties;

import dk.sosi.seal.pki.impl.HashMapCertificateCache;
import dk.sosi.seal.pki.impl.PropertiesSOSIConfiguration;
import dk.sosi.seal.pki.impl.federationcert.FederationCertificateStoreAdapter;

/**
 * Implementation of CertificationAuthority representing the OCES test CA.
 *
 * @author ads@lakeside.dk
 * @since 2.0
 */
public class OCESTestCertificationAuthority extends AbstractOCESCertificationAuthority {

	private static final String OCES3_TEST_ROOT_CERTIFICATE_BASE_64 =
			"MIIGsjCCBGagAwIBAgIUVz9X5nUw8aB3ffvGnwkEONM2AlYwQQYJKoZIhvcNAQEK"+
					"MDSgDzANBglghkgBZQMEAgMFAKEcMBoGCSqGSIb3DQEBCDANBglghkgBZQMEAgMF"+
					"AKIDAgFAMGIxJDAiBgNVBAMMG0RlbiBEYW5za2UgU3RhdCBPQ0VTIHJvZC1DQTET"+
					"MBEGA1UECwwKVGVzdCAtIGN0aTEYMBYGA1UECgwPRGVuIERhbnNrZSBTdGF0MQsw"+
					"CQYDVQQGEwJESzAeFw0yMTAxMjgwOTQ5MjVaFw00NjAxMjIwOTQ5MjRaMGIxJDAi"+
					"BgNVBAMMG0RlbiBEYW5za2UgU3RhdCBPQ0VTIHJvZC1DQTETMBEGA1UECwwKVGVz"+
					"dCAtIGN0aTEYMBYGA1UECgwPRGVuIERhbnNrZSBTdGF0MQswCQYDVQQGEwJESzCC"+
					"AiIwDQYJKoZIhvcNAQEBBQADggIPADCCAgoCggIBAL1pnHOstgHrlMGwwnUMP7I6"+
					"uEgZQkzvNcdOF3sjtJh3/i/DwdpAYTEylpf0JQrJl6lJogiukL0wDe08Xvk/NGtc"+
					"0ISWV5nLnD1Vb16RWdInyuz+g7QQepMQljFfsXHixmbjA/ICbmA+/5qDRFHK49zI"+
					"UDRnhDKxgLYGBTFoIvL7mJ3SQJfzoD8Ge0vjUbNvH479k3zmnbOZaht/l4cZcPZH"+
					"0hjdLFqgIaukwNt8Yk14KeMxqNm4KzPpikGnHtZ30xqVNdCGOetSClRB9d8Qx9Hw"+
					"55/vH5/5L9CPAlYH+2TX1l9O+i6kwleE0T7NgVRTTyJitXB7KSPbnDTKH/LVHJBH"+
					"piCQBsqxJguA9k6ujWV2G/Ko2NYkKpMvxc8lQcroMx7LxknEEX/juwkO4aRFaLWQ"+
					"AAJbeo7Av/rPZpYoCN2ri8jVtPq14Aqwx1Y66heYYZPCYMPYD6F22jNcxQE1Y1NI"+
					"GcdIkx8y7j8gP0XgpioRtbz1ej+fw+vGPW1mNQ2Cucg4ZL/o40jwTyigR3qiUa2D"+
					"pDEd1nWYN3X6N1CBz1EYCGlQPNcuQ2IponQlyRCmBQ+e9N9ouPajdcq7QjAZQE78"+
					"mhSwnF996NRnjdg5sMBqRvVR7DpNRb51LitiLJRiC+gOQy94bI4Qu5ewG5ZHHNf6"+
					"PTpZ3g5GV70DsBmnAtrhAgMBAAGjgfcwgfQwDwYDVR0TAQH/BAUwAwEB/zBzBggr"+
					"BgEFBQcBAQRnMGUwOwYIKwYBBQUHMAKGL2h0dHA6Ly9jYTEuY3RpLWdvdi5kay9v"+
					"Y2VzL3Jvb3QvY2FjZXJ0L3Jvb3QuY2VyMCYGCCsGAQUFBzABhhpodHRwOi8vY2Ex"+
					"LmN0aS1nb3YuZGsvb2NzcDA9BgNVHR8ENjA0MDKgMKAuhixodHRwOi8vY2ExLmN0"+
					"aS1nb3YuZGsvb2Nlcy9yb290L2NybC9yb290LmNybDAdBgNVHQ4EFgQUOdze3tCQ"+
					"Jkeg4MZuSX8m8qkv9hswDgYDVR0PAQH/BAQDAgEGMEEGCSqGSIb3DQEBCjA0oA8w"+
					"DQYJYIZIAWUDBAIDBQChHDAaBgkqhkiG9w0BAQgwDQYJYIZIAWUDBAIDBQCiAwIB"+
					"QAOCAgEABzNBub3P/ND9hpsevGZSm0mz0OhvkRUGU1JLKpCmaXMx4lb86gxzpj4C"+
					"6wjxWfofNCsRWlLEN52aJslFjzFm7LYvi+S9LTp+yxas7JSFXXkViaDfi2y+FiIZ"+
					"IJhkQWOV1gzkGZwjTEqhpzXnYJYTyLk/MZ1E1o0EcuxDFk2izpUFDR2TdYrCy8xQ"+
					"YRTOVdB/oW7nljdQYEzaDrrnQwanAnNkqU5Lse5DQ8P5aATF+BBL7QFQwqnMiFeY"+
					"2THu948ggl7oTuaPrFysWv9JrxWRQD9NX4pC1mgUkgJnejGOihaRKjVDcJMxCe4Y"+
					"B5tpX+JhaNO+gW4YlwxSgFAb7UG+nF8qtGCRoAZQaPzWVokYyYHSW/qLMCPySiWA"+
					"9GlMEGDuiZF/xkO96vtNA2MCJnPQCW1OJw+v5IQWWFuYXCYMNzepfiGUNB+ExYP2"+
					"wuRkMOm2EBCWt8vay2/aUEQSrWAd2BuT/nUWt7vkT7cQRtiu89axJ3SZ8P+QwDr+"+
					"zoEF82tKZJ2hXuVjVaiEwxnDmv+UuQMkl52LY3XRfa6F7zO9CoTbPyCazkp+6YWg"+
					"OdttM00ic5kscFT5kOH/0Pb6pILDWDnJ5L8XqVxb3zRXO/DM/IAwF2uV1PbLq7ly"+
					"LTiVhleQahXJ2SeRnon8KdjMiCdjlpOxHZZgEENPf08RSOQqEiI=";

	private static final String OCES3_TEST_INTERMEDIATE_CERTIFICATE_BASE_64 =
					"MIIGYTCCBBWgAwIBAgIUc01wpi5SHNF8ZEyZoJAOHVngYa4wQQYJKoZIhvcNAQEK"+
					"MDSgDzANBglghkgBZQMEAgMFAKEcMBoGCSqGSIb3DQEBCDANBglghkgBZQMEAgMF"+
					"AKIDAgFAMGIxJDAiBgNVBAMMG0RlbiBEYW5za2UgU3RhdCBPQ0VTIHJvZC1DQTET"+
					"MBEGA1UECwwKVGVzdCAtIGN0aTEYMBYGA1UECgwPRGVuIERhbnNrZSBTdGF0MQsw"+
					"CQYDVQQGEwJESzAeFw0yMTAzMTIwNzQyNTlaFw0zMTAzMTAwNzQyNThaMGsxLTAr"+
					"BgNVBAMMJERlbiBEYW5za2UgU3RhdCBPQ0VTIHVkc3RlZGVuZGUtQ0EgMTETMBEG"+
					"A1UECwwKVGVzdCAtIGN0aTEYMBYGA1UECgwPRGVuIERhbnNrZSBTdGF0MQswCQYD"+
					"VQQGEwJESzCCAaIwDQYJKoZIhvcNAQEBBQADggGPADCCAYoCggGBANOLofW5cEL4"+
					"GKGtDfVb75BI+UXI5nl/gnoi1CJcsirR+07jFP4x7mSZe/Olr0z1M3boTVaLCtdZ"+
					"urqR6AysVI9z9ymzE8zCLxE5U1x3ay/x/0J0iTxpqHqDhSF0OH+Ymb1vgp+9ta6k"+
					"PweSdG+WjJL+5xkeynGP2Zd80O2RusLAjx7s7b8sR73ImevflENJXNAJwL6MQrx4"+
					"wiTZYdPOVOSyWKTMHnBJ0MTinPyFxIv8zfbpJ2w8I+/YNTzI9EsXDDdBpQ4+asKx"+
					"A4qZY/1+OQbVTGgMTNQKl7GireR2R+cTobGiD4zzd/7mt12/gQDkjOFtSVCuOfCU"+
					"82EKilEysrnKhqFgtyGzUgRQ0PFePdI8DS50popRVtNlKKo17eE3P2GlFr9SIm9U"+
					"48GN10Eb1q+X+K1OjmNJBUVYVHF3fxC2iPySLbwkS2DSj/A6fBznj/FkIGL1BzcG"+
					"+DQPimsqKDZoIBm5SBOmi4ujoTxi+4w7h6uLyUt3D3vsPL1yUkl+qQIDAQABo4IB"+
					"HDCCARgwEgYDVR0TAQH/BAgwBgEB/wIBADAfBgNVHSMEGDAWgBQ53N7e0JAmR6Dg"+
					"xm5JfybyqS/2GzBzBggrBgEFBQcBAQRnMGUwOwYIKwYBBQUHMAKGL2h0dHA6Ly9j"+
					"YTEuY3RpLWdvdi5kay9vY2VzL3Jvb3QvY2FjZXJ0L3Jvb3QuY2VyMCYGCCsGAQUF"+
					"BzABhhpodHRwOi8vY2ExLmN0aS1nb3YuZGsvb2NzcDA9BgNVHR8ENjA0MDKgMKAu"+
					"hixodHRwOi8vY2ExLmN0aS1nb3YuZGsvb2Nlcy9yb290L2NybC9yb290LmNybDAd"+
					"BgNVHQ4EFgQUfyif2XGZQuJ159c1di5NCCVtdl4wDgYDVR0PAQH/BAQDAgEGMEEG"+
					"CSqGSIb3DQEBCjA0oA8wDQYJYIZIAWUDBAIDBQChHDAaBgkqhkiG9w0BAQgwDQYJ"+
					"YIZIAWUDBAIDBQCiAwIBQAOCAgEAuJcPSd3E6SqABGEckwvdF0W0TrScGglvOyyX"+
					"5Sx7jChPy1FFJS8vxgoqd8CYoXhzzRzuaTLLWE0P7sz51sZJKGMMA4p+f7/h4ftz"+
					"fEON1359fckE/SK5RA9IDdzVmiKxDoi8bAO/CyO/mXnZm0WFdtsU7OGgHCwAItIj"+
					"NLanSnRijazcwsNfxNJoh99S6dAV/GGzo9NMPIRrLNG3IZboPbQe07MT/DFeHP7/"+
					"xA5KxB2SdnPTfFevulJOARq4Fv+rThaSp+j67AhmzI6qrIj8r4GlCyr3+EI474ie"+
					"ZeiiClMtT+DbHpPPfDMvNk1DYOqWNRi4warsDXhVkhXdGZrAP6OE3hnVbS94q5B7"+
					"FhwugFl5XzSu06F3s3BLHrEbMCff2QZuGtf2xZIMRlWO8tivguxM2KLJYewGbOuM"+
					"uk1SQcB22fF0NQH2yzqnW6DFTGORDgYfy8IJItv82IQxKiS7/FMaoMp3mLyqb9dN"+
					"rgRyvnxwMuzLiVBDSJX0Hf78Nx/MwFvdbLawzdi9Fro9EWJ1hzjfJMZTiCP7my/E"+
					"5UJMj/Sjce3jLep25c4pu8qrY7PC6o4OqICOkLJAVNL1TU+OVpKnUd7flWBOA4rH"+
					"aR0dZY+OC/ZQhymMgT6Zg+Yj6d3cNwh8tiJTlCUYSlX27NP2HGhjZ5OlyHcOXtvQ"+
					"3VlJhI4=";
	public static final List<X509Certificate> OCES_3_TEST_ROOT_CERTIFICATES = asCertificates(OCES3_TEST_ROOT_CERTIFICATE_BASE_64, OCES3_TEST_ROOT_CERTIFICATE_BASE_64);

	public static final X509Certificate OCES_3_TEST_INTERMEDIATE_CERTIFICATE = asCertificate(OCES3_TEST_INTERMEDIATE_CERTIFICATE_BASE_64);

	/**
	 * Constructor for the <code>OCESTestCertificationAuthority</code> class.
	 *
	 * @param properties
	 *            The initialization <code>Properties</code> of the system
	 * @deprecated will be removed in future Seal release
	 */
	@Deprecated
	public OCESTestCertificationAuthority(Properties properties, CertificateStatusChecker certificateStatusChecker, IntermediateCertificateCache intermediateCertificateCache) {
		// This is not nice at all! To be removed in next Seal release
		this(PropertiesSOSIConfiguration.createWithDefaultOcesTestProperties(properties), certificateStatusChecker, intermediateCertificateCache,
				new FederationCertificateStoreAdapter(PropertiesSOSIConfiguration.createWithDefaultOcesTestProperties(properties), new HashMapCertificateCache()));
	}

	public OCESTestCertificationAuthority(SOSIConfiguration configuration, CertificateStatusChecker certificateStatusChecker, IntermediateCertificateCache intermediateCertificateCache,
			FederationCertificateResolver federationCertificateResolver) {
		super(configuration, certificateStatusChecker, intermediateCertificateCache, federationCertificateResolver);
	}

	@Override
	protected List<X509Certificate> getOCES3RootCertificates() {
		return OCES_3_TEST_ROOT_CERTIFICATES;
	}

	protected String getCertificationAuthorityName() {
		return "OCES Test";
	}

}