package dk.sosi.seal.security;

import org.jcp.xml.dsig.internal.dom.DOMX509IssuerSerial;
import org.junit.Test;
import org.mockito.Mockito;

import javax.xml.crypto.dsig.keyinfo.X509IssuerSerial;
import java.math.BigInteger;
import java.security.Principal;
import java.security.cert.X509Certificate;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import static org.mockito.Mockito.when;

public class HolderOfKeyTest {

    @Test
    public void holderOfKeyWithCertificateMatchesSameCertificate() {
        X509Certificate certificate = Mockito.mock(X509Certificate.class);
        HolderOfKey holderOfKey = new HolderOfKey(certificate);

        assertTrue(holderOfKey.matches(certificate));
    }

    @Test
    public void holderOfKeyWithCertificateDoesNotMatchDifferentCertificate() {
        X509Certificate certificate = Mockito.mock(X509Certificate.class);
        X509Certificate differentCertificate = Mockito.mock(X509Certificate.class);
        HolderOfKey holderOfKey = new HolderOfKey(certificate);

        assertFalse(holderOfKey.matches(differentCertificate));
    }

    @Test
    public void holderOfKeyMatchesWithSameCertificateUsingDistinguishedName() {
        X509Certificate certificate = Mockito.mock(X509Certificate.class);
        Principal issuerDN = Mockito.mock(Principal.class);
        String issuerName = "C=DK, O=Den Danske Stat, OU=Test - cti, CN=Den Danske Stat OCES udstedende-CA 1";
        when(issuerDN.getName()).thenReturn(issuerName);
        when(certificate.getIssuerDN()).thenReturn(issuerDN);
        when(certificate.getSerialNumber()).thenReturn(new BigInteger("12345"));

        HolderOfKey holderOfKey = new HolderOfKey(certificate);

        assertTrue(holderOfKey.matches(certificate));
    }

    @Test
    public void holderOfKeyDoesNotMatchWithDifferentCertificateUsingDistinguishedName() {
        X509Certificate certificate1 = Mockito.mock(X509Certificate.class);
        Principal issuerDN1 = Mockito.mock(Principal.class);
        String issuerName1 = "C=DK, O=Den Danske Stat, OU=Test - cti, CN=Den Danske Stat OCES udstedende-CA 1";
        when(issuerDN1.getName()).thenReturn(issuerName1);
        when(certificate1.getIssuerDN()).thenReturn(issuerDN1);
        when(certificate1.getSerialNumber()).thenReturn(new BigInteger("12345"));

        X509Certificate certificate2 = Mockito.mock(X509Certificate.class);
        Principal issuerDN2 = Mockito.mock(Principal.class);
        String issuerName2 = "C=DK, O=Den Danske Stat, OU=Test - cti, CN=Den Danske Stat OCES udstedende-CA 2";
        when(issuerDN2.getName()).thenReturn(issuerName2);
        when(certificate2.getIssuerDN()).thenReturn(issuerDN2);
        when(certificate2.getSerialNumber()).thenReturn(new BigInteger("67890"));

        HolderOfKey holderOfKey = new HolderOfKey(certificate1);

        assertFalse(holderOfKey.matches(certificate2));
    }

    @Test
    public void holderOfKeyMatchesWithCertificateHavingSameIssuerSerial() {
        X509IssuerSerial issuerSerial = new DOMX509IssuerSerial("C=DK, O=Den Danske Stat, OU=Test - cti, CN=Den Danske Stat OCES udstedende-CA 1", new BigInteger("12345"));
        X509Certificate certificate = Mockito.mock(X509Certificate.class);
        Principal issuerDN = Mockito.mock(Principal.class);
        String issuerName = "C=DK, O=Den Danske Stat, OU=Test - cti, CN=Den Danske Stat OCES udstedende-CA 1";
        when(issuerDN.getName()).thenReturn(issuerName);
        when(certificate.getIssuerDN()).thenReturn(issuerDN);
        when(certificate.getSerialNumber()).thenReturn(new BigInteger("12345"));

        HolderOfKey holderOfKey = new HolderOfKey(issuerSerial);

        assertTrue(holderOfKey.matches(certificate));
    }

    @Test
    public void matchesWithCertificateHavingIdenticalIssuerSerialDespiteSpacing() {
        X509IssuerSerial issuerSerial = new DOMX509IssuerSerial("C=DK, O=Den Danske Stat, OU=Test - cti, CN=Den Danske Stat OCES udstedende-CA 1", new BigInteger("12345"));
        X509Certificate certificate = Mockito.mock(X509Certificate.class);
        Principal issuerDN = Mockito.mock(Principal.class);
        String issuerName = "C= DK, O= Den Danske Stat, OU= Test - cti, CN= Den Danske Stat OCES udstedende-CA 1";
        when(issuerDN.getName()).thenReturn(issuerName);
        when(certificate.getIssuerDN()).thenReturn(issuerDN);
        when(certificate.getSerialNumber()).thenReturn(new BigInteger("12345"));

        HolderOfKey holderOfKey = new HolderOfKey(issuerSerial);

        assertTrue(holderOfKey.matches(certificate));
    }

    @Test
    public void matchesWithCertificateHavingIdenticalIssuerSerialWithReorderedDNComponents() {
        X509IssuerSerial issuerSerial = new DOMX509IssuerSerial("C=DK, O=Den Danske Stat, OU=Test - cti, CN=Den Danske Stat OCES udstedende-CA 1", new BigInteger("12345"));
        X509Certificate certificate = Mockito.mock(X509Certificate.class);
        Principal issuerDN = Mockito.mock(Principal.class);
        String issuerName = "C=DK, OU=Test - cti, O=Den Danske Stat, CN=Den Danske Stat OCES udstedende-CA 1";
        when(issuerDN.getName()).thenReturn(issuerName);
        when(certificate.getIssuerDN()).thenReturn(issuerDN);
        when(certificate.getSerialNumber()).thenReturn(new BigInteger("12345"));

        HolderOfKey holderOfKey = new HolderOfKey(issuerSerial);

        assertTrue(holderOfKey.matches(certificate));
    }

    @Test
    public void matchesWithCertificateHavingIdenticalIssuerSerialWithDifferentDNOrdering() {
        X509IssuerSerial issuerSerial = new DOMX509IssuerSerial("O=Den Danske Stat, OU=Test - cti, CN=Den Danske Stat OCES udstedende-CA 1, C=DK", new BigInteger("12345"));
        X509Certificate certificate = Mockito.mock(X509Certificate.class);
        Principal issuerDN = Mockito.mock(Principal.class);
        String issuerName = "C=DK, OU=Test - cti, O=Den Danske Stat, CN=Den Danske Stat OCES udstedende-CA 1";
        when(issuerDN.getName()).thenReturn(issuerName);
        when(certificate.getIssuerDN()).thenReturn(issuerDN);
        when(certificate.getSerialNumber()).thenReturn(new BigInteger("12345"));

        HolderOfKey holderOfKey = new HolderOfKey(issuerSerial);

        assertTrue(holderOfKey.matches(certificate));
    }
}
